from .cfg import cfg


# --------------------
## holds functions for mode facets
class ModeBaseFacet:
    # --------------------
    ## constructor
    def __init__(self):
        ## holds reference to the format facet
        self.format = None

    # --------------------
    ## save any lines.
    # currently ignored in "null" and "ut/mock" modes
    #
    # @return None
    def save(self):
        pass

    # --------------------
    ## any additional termination steps to do
    #
    # @return None
    def term_step2(self):
        pass

    # --------------------
    ## set the format facet based on the current format selected
    #
    # @return None
    def set_log_it_fn(self):
        if cfg.log_format == cfg.log_format_elapsed:
            from .format_elapsed_facet import FormatElapsedFacet
            self.format = FormatElapsedFacet()
        elif cfg.log_format == cfg.log_format_prefix:
            from .format_prefix_facet import FormatPrefixFacet
            self.format = FormatPrefixFacet()
        elif cfg.log_format == cfg.log_format_none:
            from .format_none_facet import FormatNoneFacet
            self.format = FormatNoneFacet()
        else:  # pragma: no cover
            cfg.abort(f'in ModeFacetUt.set_log_it_fn: unhandled log format {cfg.log_format}')

    # --------------------
    ## prevents lint/IDE warnings; ignored in all modes except UT
    # @return no lines
    @property
    def ut_lines(self):
        return []

    # --------------------
    ## prevents lint/IDE warnings; ignored in all modes except UT
    # @return none
    def ut_clear(self):  # pragma: no cover
        pass



from .cfg import cfg
from .mode_base_facet import ModeBaseFacet


# --------------------
## holds functions for mode "ut" or "mock"
class ModeUtFacet(ModeBaseFacet):
    # --------------------
    ## constructor
    def __init__(self):
        super().__init__()

        ## holds lines during UTs; must be a public attribute
        self._ut_lines = []
        ## holds the current dots in the line
        self._dot_line = ''

    # no save() needed
    # no term_step2() needed

    # --------------------
    ## return current set of UT lines
    # @return current set of UT lines
    @property
    def ut_lines(self):
        return self._ut_lines

    # --------------------
    ## clear out current UT lines
    # @return none
    def ut_clear(self):
        self._ut_lines = []

    # --------------------
    ## callback function to log a line in immediate mode
    #
    # @param info   the line info
    # @return None
    def log_it(self, info):
        if not info[0] and not info[1]:  # !always_print and !verbose
            return

        line1 = self.format.build_line(info)
        full_line = self.format.add_color_user_pfx(info, line1)
        self._print_line(info, full_line)

    # --------------------
    ## print a line in immediate mode with the given colors and user_prefixes.
    # it is printed to stdout, which is then flushed
    #
    # @param info        the line info
    # @param full_line   the text of the line
    # @return None
    def _print_line(self, info, full_line):
        if info[3] == '.' and info[4] == (None,):
            self._handle_dots(full_line)
        else:
            # last call was a dot, so reset and print a newline ready for the new log line
            self._reset_dots()
            self._ut_lines.append(full_line)

    # --------------------
    ## If past max_dots, print a newline. Then print a dot (may have color codes).
    #
    # @param line   dot with or without color codes
    # @return none
    def _handle_dots(self, line):
        if cfg.dots >= cfg.max_dots:
            self._ut_lines.append(self._dot_line)
            self._dot_line = ''
            cfg.dots = 0

        self._dot_line += line
        cfg.dots += 1

    # --------------------
    ## reset the dots count and print a newline as needed
    #
    # @return None
    def _reset_dots(self):
        if cfg.dots != 0:
            self._ut_lines.append(self._dot_line)
            self._dot_line = ''
            cfg.dots = 0



import time
from datetime import datetime

from .cfg import cfg


# --------------------
## holds common functions for format facets
class FormatBaseFacet:
    # --------------------
    ## add color and user prefix if any to the given line
    #
    # @param info   the logging info
    # @param line1  the current line text
    # @return the full line text, should be ready to print
    def add_color_user_pfx(self, info, line1):
        # add colors and user prefix if any
        color_pfx, color_sfx = self._get_color(info[6])
        if info[5] is None:
            user_pfx = ''
        else:
            user_pfx = f'{info[5]: <4} '

        full_line = f'{color_pfx}{user_pfx}{line1}{color_sfx}'
        return full_line

    # --------------------
    ## get color code required for the given color.
    # if it is not a recognized color, the value given is used for the code.
    # if the color code is empty, then no color code is returned ('').
    #
    # @param color   the color to use or the color code to use
    # @return the color code prefix, the color code suffix
    def _get_color(self, color):
        clr_sfx = '\033[0m'
        if color == 'green':
            clr_pfx = '\033[92m'
        elif color == 'yellow':
            clr_pfx = '\033[33m'
        elif color == 'red':
            clr_pfx = '\033[31m'
        elif color == 'blue':
            clr_pfx = '\033[1;36m'
        elif color is not None:
            clr_pfx = color
        else:
            clr_pfx = ''
            clr_sfx = ''

        return clr_pfx, clr_sfx

    # --------------------
    ## get the text for a full DTS line.
    #
    # @param dts     the current datetime value
    # @return the dts text
    def get_full_dts(self, dts):
        # restart the timer; user wants the full DTS and elapsed is since that absolute time
        cfg.start_time = dts

        cfg.save_config()
        dts_str = time.strftime("%Y/%m/%d", time.localtime(cfg.start_time))
        t_str = datetime.fromtimestamp(cfg.start_time).strftime('%H:%M:%S.%f')[:12]
        full_dts = f'{"DTS": <4} {dts_str} {t_str}'
        if cfg.log_format == cfg.log_format_elapsed:
            full_dts = f'{"": <9} {full_dts}'
        return full_dts



from datetime import timedelta

from .cfg import cfg
from .format_base_facet import FormatBaseFacet


# --------------------
## functions for format "elapsed"
class FormatElapsedFacet(FormatBaseFacet):
    # --------------------
    ## build a line text based on the given info. Use "elapsed" format.
    #
    # @param info   the line info
    # @return the full text of the line
    def build_line(self, info):
        dts = info[2]
        elapsed = dts - cfg.start_time
        if elapsed >= 3600.0:
            cfg.start_time = dts
            elapsed = 0
        t_str = self._get_elapsed_str(elapsed)

        prefix = info[3]
        if prefix is None and info[4] == (None,):
            full_line = self.get_full_dts(info[2])
        elif info[3] == '.' and info[4] == (None,):
            full_line = '.'
        elif prefix is None:  # raw line
            full_line = ' '.join(map(str, info[4]))
        else:
            line = ' '.join(map(str, info[4]))
            full_line = f'{t_str} {prefix: <4} {line}'
        return full_line

    # --------------------
    ## generate the string of the given elapsed time
    #
    # @param elapsed   the elapsed time to format
    # @return the string ("MM.SS.nnn")
    def _get_elapsed_str(self, elapsed):
        t_str = timedelta(seconds=elapsed)
        # rare case: str(timedelta) makes the ".000000" optional if the number of microseconds is 0
        if t_str.microseconds == 0:  # pragma: no cover
            # bump the number of microseconds by 1 to make sure the full string is formatted
            t_str = timedelta(seconds=elapsed + 0.000001)
        return str(t_str)[2:11]
